/*
 * SPDX-FileCopyrightText: syuilo and other misskey contributors
 * SPDX-License-Identifier: AGPL-3.0-only
 */

import bcrypt from 'bcryptjs';
import { Inject, Injectable } from '@nestjs/common';
import type { UsersRepository, UserProfilesRepository } from '@/models/_.js';
import { Endpoint } from '@/server/api/endpoint-base.js';
import { TruncateAccountService } from '@/core/TruncateAccountService.js';
import { DI } from '@/di-symbols.js';
import { UserAuthService } from '@/core/UserAuthService.js';
import {ApiError} from "@/server/api/error.js";

export const meta = {
	requireCredential: true,

	secure: true,
	requireRolePolicy: 'canUseAccountTruncate',

	errors: {
		authenticationFailed: {
			message: 'Your password or 2FA token is invalid.',
			code: 'AUTHENTICATION_FAILED',
			id: 'ea791cff-63e7-4b2a-92fc-646ab641794e',
		},
		alreadyRemoved: {
			message: 'Your account is removed.',
			code: 'ACCOUNT_REMOVED',
			id: '59b8f0e6-6eb2-4dc1-a080-1de3108416d0',
		},
	},
} as const;

export const paramDef = {
	type: 'object',
	properties: {
		password: { type: 'string' },
		token: { type: 'string', nullable: true },
		purgeDrive: { type: 'boolean', nullable: true },
	},
	required: ['password'],
} as const;

@Injectable()
export default class extends Endpoint<typeof meta, typeof paramDef> { // eslint-disable-line import/no-default-export
	constructor(
		@Inject(DI.usersRepository)
		private usersRepository: UsersRepository,

		@Inject(DI.userProfilesRepository)
		private userProfilesRepository: UserProfilesRepository,

		private userAuthService: UserAuthService,
		private truncateAccountService: TruncateAccountService,
	) {
		super(meta, paramDef, async (ps, me) => {
			const purgeDrive = !!ps.purgeDrive;
			const profile = await this.userProfilesRepository.findOneByOrFail({ userId: me.id });

			const userDetailed = await this.usersRepository.findOneByOrFail({ id: me.id });
			if (userDetailed.isDeleted) {
				throw new ApiError(meta.errors.alreadyRemoved);
			}

			const passwordMatched = await bcrypt.compare(ps.password, profile.password!);
			if (!passwordMatched) {
				throw new ApiError(meta.errors.authenticationFailed);
			}

			if (profile.twoFactorEnabled) {
				const token = ps.token;
				if (token == null) {
					throw new ApiError(meta.errors.authenticationFailed);
				}

				await this.userAuthService.twoFactorAuthenticate(profile, token);
			}

			await this.truncateAccountService.truncateAccount(me, purgeDrive);
		});
	}
}
